<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Resources\BillResource;
use App\Models\Bill;
use App\Models\BillDetail;
use App\Models\Item;
use App\Models\ParcelDetail;
use App\Models\Recipe;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class BillController extends Controller
{
    public function index() {
        $bills = $this->paginateOrGet(
            Bill::with([
                "billDetails.recipe.recipeDetails.item",
                "parcelDetails.item",
                "user",
                "franchise"
            ])->filter()->latest());

        return BillResource::collection($bills);
    }

    public function store(Request $request) {
        $data = $request->validate([
            'franchise_id' => ['required', "exists:franchises,id"],
            'payment_type_id' => ['required', "exists:payment_types,id"],
            'order_source_id' => ['sometimes', "exists:order_sources,id"],
            'table_id' => ['sometimes', "exists:tables,id"],

            'amount' => ['required', "numeric", "between:0,99999999.99"],
            'discount' => ['sometimes', "numeric", "between:0,99999999.99"],
            'gst' => ['sometimes', "string", "max:20"],
            'is_parcel' => ['required', 'boolean'],
            'is_active' => ["required", 'boolean'],

            'name' => ["required", "string", "max:254"],
            'email' => ["required", "email"],
            'mobile_no' => ['required', "string", "max:20"],

            "bill_details" => ["required", "array"],
            "bill_details.*.recipe_id" => ["required", "exists:recipes,id"],
            "bill_details.*.qty" => ["required", "numeric", "between:0,65534"],
            "bill_details.*.price" => ["required", "numeric", "between:0,99999999.99"],
            "bill_details.*.total_amount" => ["required", "numeric", "between:0,99999999.99"],

            "parcel_details" => ["sometimes", "array"],
        ]);


        return DB::transaction(function () use ($data) {
            $bill = new Bill();
            $user = User::query()->where("email", $data["email"])
                ->orWhere("mobile_no", $data["mobile_no"])->first();

            $bill->fill([
                "franchise_id" => $data["franchise_id"],
                'payment_type_id' => $data["payment_type_id"],
                'order_source_id' => $data['order_source_id'],
                'amount' => $data['amount'],
                'discount' => $data['discount'] ?? null,
                'gst' => $data['gst'] ?? null
            ]);

            if (!isset($user)) {
                $user = User::query()->create([
                    "name" => $data["name"],
                    'email' => $data['email'],
                    'franchise_id' => $data['franchise_id'],
                    'address' => isset($data['address']) ?: null,
                    'password' => 'password',
                    'role' => 'customer',
                    'is_customer' => 1,
                    'mobile_no' => $data['mobile_no']
                ]);
            }
            $bill->user_id = $user->id;
            $bill->save();

            if (isset($data["bill_details"])) {
                foreach ($data["bill_details"] as $bill_detail) {
                    $billDetail = new BillDetail($bill_detail);
                    $billDetail->bill_id = $bill->id;
                    $billDetail->save();

                    $recipe = Recipe::query()->findOrFail($billDetail->recipe_id);
                    $recipeDetails = $recipe->recipeDetails();

                    foreach ($recipeDetails as $recipeDetail) {
                        $item = $recipeDetail->item;
                        if ($item->qty >= ($recipeDetail->qty * $bill_detail["qty"])) {
                            $item->qty -= ($recipeDetail->qty * $bill_detail["qty"]);
                            $item->save();
                        } else {
                            return response()->json(["error" => "Insufficient qty for item - ".$item->name], 422);
                        }
                    }

                }
            }

            if (isset($data["parcel_details"])) {
                // TODO: ask
                // $bill->is_parcel = true;
                foreach ($data["parcel_details"] as $parcel_detail) {
                    $parcelDetail = new ParcelDetail($parcel_detail);
                    $parcelDetail->bill_id = $bill->id;

                    $item = Item::findOrFail($parcelDetail["item_id"]);

                    if ($item->qty >= $parcelDetail["qty"]) {
                        $item->qty -= $parcel_detail["qty"];
                        $item->save();
                    } else {
                        return response()->json(["error" => "Insufficient qty for item - $item->name"], 422);
                    }
                }
            }

            // TODO: what about table?

            return new BillResource($bill);
        });
    }

    public function show(Bill $bill) {
        return new BillResource($bill->loadMissing([
            "billDetails.recipe.recipeDetails.item",
            "parcelDetails.item",
            "user",
            "franchise"
        ]));
    }

    public function update(Request $request, Bill $bill) {
        $data = $request->validate([
            'franchise_id' => ['sometimes', "exists:franchises,id"],
            'payment_type_id' => ['sometimes', "exists:payment_types,id"],
            'order_source_id' => ['sometimes', "exists:order_sources,id"],
            'table_id' => ['sometimes', "exists:tables,id"],

            'amount' => ['sometimes', "numeric", "between:0,99999999.99"],
            'discount' => ['sometimes', "numeric", "between:0,99999999.99"],
            'gst' => ['sometimes', "string", "max:20"],
            'is_parcel' => ['sometimes', 'boolean'],
            'is_active' => ["sometimes", 'boolean'],

            'name' => ["sometimes"],
            'email' => ["sometimes"],
            'mobile_no' => ['sometimes'],
        ]);

        return DB::transaction(function () use ($bill, $request) {
            $bill->fill($request->except(
                "name", "email", "mobile_no",
                "bill_details", "parcel_details",
                "delete_bill_details", "delete_parcel_details"
            ));

            if ($request->has("bill_details")) {
                foreach ($request->bill_details as $bill_detail) {
                    $billDetail = new BillDetail($bill_detail);
                    $billDetail->bill_id = $bill->id;

                    $recipe = Recipe::findOrFail($bill_detail["recipe_id"]);
                    $recipeDetails = $recipe->recipeDetails();

                    foreach ($recipeDetails as $recipeDetail) {
                        $item = $recipeDetail->item;
                        if ($item->qty >= ($recipeDetail->qty * $bill_detail["qty"])) {
                            $item->qty -= ($recipeDetail->qty * $bill_detail["qty"]);
                            $item->save();
                        } else {
                            return response()->json(["error" => "Insufficient qty for item - ".$item->name], 422);
                        }
                    }
                }
            }

            if ($request->has("delete_bill_details")) {
                foreach ($request->delete_bill_details as $bill_detail_id) {
                    $billDetail = BillDetail::query()->findOrFail($bill_detail_id);
                    $recipe = Recipe::query()->findOrFail($billDetail->recipe_id);

                    $recipeDetails = $recipe->recipeDetails();
                    foreach ($recipeDetails as $recipeDetail) {
                        $item = $recipeDetail->item;
                        $item->qty += ($recipeDetail->qty * $billDetail->qty);
                        $item->save();
                    }
                    $billDetail->delete();
                }
            }

            if ($request->has("delete_parcel_details")) {
                foreach ($request->delete_parcel_details as $parcel_detail_id) {
                    $parcelDetail = ParcelDetail::query()->findOrFail($parcel_detail_id);

                    $item = $parcelDetail->item;
                    $item->qty += $parcelDetail->qty;
                    $item->save();

                    $parcelDetail->delete();
                }
            }

            return $bill;
        });
    }

    public function destroy(Bill $bill) {
        $bill->delete();

        return response()->json(["message" => "Bill deleted successfully!"]);
    }

    public function offlineBills(Request $request) {
        return DB::transaction(function () use ($request) {
            foreach ($request->bills as $bill) {
                $user = User::query()->firstOrCreate(
                    ["email" => $bill['email']],
                    [
                        'name' => $bill['name'],
                        'franchise_id' => $bill['franchise_id'],
                        'mobile_no' => $bill['mobile_no'],
                        'password' => 'password', // You may want to set a default password
                        'role' => 'customer',
                        'is_customer' => true,
                    ]
                );

                $newBill = Bill::query()->create([
                    'franchise_id' => $bill["franchise_id"],
                    'payment_type_id' => $bill["payment_type_id"],
                    'order_source_id' => $bill["order_source_id"],
                    'amount' => $bill["amount"],
                    'discount' => $bill["discount"],
                    'gst' => $bill["gst"],
                ]);

                $newBill->user()->associate($user);
                $newBill->save();

                if (!isset($bill["bill_details"])) {
                    foreach ($bill["bill_details"] as $bill_detail) {
                        $billDetail = new BillDetail($bill_detail);
                        $billDetail->recipe_id = $bill_detail["recipe_id"];
                        $billDetail->bill_id = $newBill->id;
                        $billDetail->save();
                    }
                }

                if (!isset($bill["parcel_details"])) {
                    foreach ($bill["parcel_details"] as $parcel_detail) {
                        $parcelDetail = new ParcelDetail($parcel_detail);
                        $parcelDetail->bill_id = $newBill->id;
                        $parcelDetail->save();
                    }
                }
            }

            return response()->json(["message" => "Bills processed successfully"]);
        });

    }
}
