<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Resources\InventoryOrderResource;
use App\Models\InventoryOrder;
use Illuminate\Http\Request;

class InventoryOrderController extends Controller
{
    protected array $relations = ["franchise", "inventoryOrderItems"];

    /**
     * Display a listing of the inventory orders.
     */
    public function index() {
        $inventoryOrders = $this->paginateOrGet(
            InventoryOrder::filter()->latest());

        return InventoryOrderResource::collection($inventoryOrders);
    }

    /**
     * Store a newly created inventory order in storage.
     */
    public function store(Request $request) {
        $request->validate([
            "franchise_id" => "required|exists:franchises,id",

            "for" => "required|in:".implode(",", InventoryOrder::CATEGORIES),
            "is_paid" => "required|boolean",
            "total" => "required|numeric|between:0,99999999.99",
            "status" => "required|string|in:".implode(",", InventoryOrder::STATUS),

            "order_items" => "required|array|min:1",
            "order_items.*.item_id" => "required|exists:items,id",
            "order_items.*.ordered_quantity" => "required|numeric|between:1,65535",
            "order_items.*.price" => "required|numeric|between:0,99999999.99"
        ]);

        // determine delivery date
        $deliverDate = now()->hour < 16 ? now()->addDay() : now()->addDays(2);

        // create order
        $inventoryOrder = InventoryOrder::query()->create(
            $request->all() + ["order_date" => now(), "delivery_date" => $deliverDate]);

        // create order_items
        $inventoryOrder->inventoryOrderItems()->createMany($request->input("order_items"));

        return new InventoryOrderResource($inventoryOrder);
    }

    /**
     * Display the specified inventory order.
     */
    public function show(InventoryOrder $inventoryOrder) {
        $inventoryOrder->loadMissing("inventoryOrderItems");

        return new InventoryOrderResource($inventoryOrder);
    }

    /**
     * Update the specified inventory order in storage.
     */
    public function update(Request $request, InventoryOrder $inventoryOrder) {
        // Validate incoming request data
        $validatedData = $request->validate([
            "franchise_id" => "sometimes|exists:franchises,id",
            "for" => "sometimes|in:".implode(",", InventoryOrder::CATEGORIES),
            "order_date" => "sometimes|date",
            "delivery_date" => "sometimes|date|after_or_equal:order_date",
            "is_paid" => "sometimes|boolean",
            "total" => "sometimes|numeric|between:0,99999999.99",
            "status" => "required|in:".implode(",", InventoryOrder::STATUS),

            "order_items" => "sometimes|array|min:1",
            "order_items.*.id" => "nullable|exists:inventory_order_items,id", // in case of updating existing items
            "order_items.*.item_id" => "required|exists:items,id",
            "order_items.*.ordered_quantity" => "required|numeric|between:1,65535",
            "order_items.*.received_quantity" => "required|numeric|between:0,65535",
            "order_items.*.price" => "required|numeric|between:0,99999999.99",
        ]);

        // update InventoryOrder with validated data
        $inventoryOrder->update($validatedData);

        // handle inventory order items
        if ($request->filled('order_items')) {
            // extract order item IDs to keep
            $orderItemIds = collect($validatedData['order_items'])->map(function ($item) use ($inventoryOrder) {
                // update or create each inventory order item
                $orderItem = $inventoryOrder->inventoryOrderItems()
                    ->updateOrCreate(['id' => $item['id'] ?? null], $item);

                // update the item quantity for the associated item, only if received_quantity > 0
                if ($orderItem->received_quantity > 0) {
                    $orderItem->item()->increment('qty', $orderItem->received_quantity);
                }

                return $orderItem->id;
            });

            // Delete order items not present in the current request
            $inventoryOrder->inventoryOrderItems()->whereNotIn('id', $orderItemIds)->delete();
        }

        return new InventoryOrderResource($inventoryOrder);
    }

    /**
     * Remove the specified inventory order from storage.
     */
    public function destroy(InventoryOrder $inventoryOrder) {
        $inventoryOrder->delete();

        return response(["message" => "Inventory order deleted successfully!"]);
    }
}
