<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Resources\InventoryReturnResource;
use App\Models\InventoryOrder;
use App\Models\InventoryReturn;
use Illuminate\Http\Request;

class InventoryReturnController extends Controller
{
    protected array $relations = ["franchise", "inventoryReturnItems"];

    /**
     * Display a listing of the inventory returns.
     */
    public function index() {
        $inventoryReturns = $this->paginateOrGet(
            InventoryReturn::filter()->latest());

        return InventoryReturnResource::collection($inventoryReturns);
    }

    /**
     * Store a newly created inventory return in storage.
     */
    public function store(Request $request) {
        $validatedData = $request->validate([
            "franchise_id" => "required|exists:franchises,id",
            "return_of" => "required|in:".implode(",", InventoryOrder::CATEGORIES),
            "date" => "required|date",

            "return_items" => "required|array|min:1",
            "return_items.*.item_id" => "required|exists:items,id",
            "return_items.*.quantity" => "required|numeric|between:1,65535",
            "return_items.*.is_accepted" => "nullable|boolean",
        ]);

        // create the inventory return
        $inventoryReturn = InventoryReturn::query()->create($validatedData);

        // create return items in bulk
        $inventoryReturn->inventoryReturnItems()->createMany($validatedData["return_items"]);

        return new InventoryReturnResource($inventoryReturn);
    }

    /**
     * Display the specified inventory return.
     */
    public function show(InventoryReturn $inventoryReturn) {
        $inventoryReturn->loadMissing("inventoryReturnItems");

        return new InventoryReturnResource($inventoryReturn);
    }

    /**
     * Update the specified inventory return in storage.
     */
    public function update(Request $request, InventoryReturn $inventoryReturn) {
        // Validate the request data
        $validatedData = $request->validate([
            "franchise_id" => "sometimes|exists:franchises,id",
            "return_of" => "sometimes|in:".implode(",", InventoryOrder::CATEGORIES),
            "date" => "sometimes|date",

            "return_items" => "sometimes|array|min:1",
            "return_items.*.id" => "nullable|exists:inventory_return_items,id",
            "return_items.*.item_id" => "required|exists:items,id",
            "return_items.*.quantity" => "required|numeric|between:1,65535",
            "return_items.*.is_accepted" => "required|boolean",
        ]);

        // update the inventory return
        $inventoryReturn->update($validatedData);

        // update return items if provided
        if ($request->filled('return_items')) {
            // extract return item IDs to keep
            $returnItemIds = collect($validatedData['return_items'])->map(function ($item) use ($inventoryReturn) {
                // update or create each inventory return item
                $returnItem = $inventoryReturn->inventoryReturnItems()
                    ->updateOrCreate(['id' => $item['id'] ?? null], $item);

                // update the item quantity, if return item is accepted
                if ($returnItem->is_accepted) {
                    $returnItem->item()->decrement('qty', $returnItem->quantity);
                }

                return $returnItem->id;
            });

            // delete removed items
            $inventoryReturn->inventoryReturnItems()->whereNotIn('id', $returnItemIds)->delete();
        }

        return new InventoryReturnResource($inventoryReturn);
    }

    /**
     * Remove the specified inventory return from storage.
     */
    public function destroy(InventoryReturn $inventoryReturn) {
        $inventoryReturn->delete();

        return response(["message" => "Inventory return deleted successfully!"]);
    }
}
